@@ -3,7 +3,7 @@ use strict;
 use warnings;
 
 use vars qw($VERSION);
-$VERSION = '0.21_03';
+$VERSION = '0.25_03';
 
 use Carp;
 
@@ -11,7 +11,7 @@ sub DESTROY { }
 
 =head1 NAME
 
-CPAN::PackageDetails::Entries - Handle the collections of records of 02packages.details.txt.gz
+CPAN::PackageDetails::Entries - Handle the collection of records of 02packages.details.txt.gz
 
 =head1 SYNOPSIS
 
@@ -160,7 +160,21 @@ sub add_entry
 	
 	unless( defined $args{'package name'} )
 		{
-		carp "No 'package name' parameter!";
+		croak "No 'package name' parameter!";
+		return;
+		}
+
+	unless( $args{'package name'} =~ m/
+		^
+		[A-Za-z0-9_]+
+		(?: 
+			(?:\::|')
+			[A-Za-z0-9_]+
+		)*
+		\z
+		/x )
+		{
+		croak "Package name [$args{'package name'}] looks suspicious. Not adding it!";
 		return;
 		}
 		
@@ -225,11 +239,11 @@ Once called, it caches its result until you add more entries.
 
 =cut
 
+sub VERSION_PM () { 9 }
 sub as_unique_sorted_list
 	{
 	my( $self ) = @_;
 
-	
 	unless( ref $self->{sorted} eq ref [] )
 		{
 		$self->{sorted} = [];
@@ -239,15 +253,29 @@ sub as_unique_sorted_list
 		my( $k1, $k2 ) = ( $self->columns )[0,1];
 
 		my $e = $self->entries;
-		
-	# We only want the latest versions of everything:
+
+		# We only want the latest versions of everything:
 		foreach my $package ( sort keys %$e )
 			{
-			my( $highest_version ) = 
-				sort { $e->{$package}{$b} <=> $e->{$package}{$b} }
-				keys %{ $e->{$package} };
-			
-			push @{ $self->{sorted} }, $e->{$package}{$highest_version};
+			my $entries = $e->{$package};
+			eval {
+				eval { require version } or die "Could not load version.pm!";
+				die "Your version of the version module doesn't handle the parse method!"
+					unless version->can('parse');
+				} or croak( {
+					message         => $@,
+					have_version    => eval { version->VERSION },
+					need_version    => 0.74,
+					inc             => [ @INC ],
+					error           => VERSION_PM,
+					} 
+				);
+				
+			my( $highest_version ) =
+				sort { version->parse($b) <=> version->parse($a) }
+				keys %$entries;
+
+			push @{ $self->{sorted} }, $entries->{$highest_version};
 			}
 		}
 	
@@ -259,6 +287,77 @@ sub as_unique_sorted_list
 	return $return;
 	}
 
+=item get_entries_by_package( PACKAGE )
+
+Returns the entry objects for the named PACKAGE.
+
+=cut
+
+sub get_entries_by_package
+	{
+	my( $self, $package ) = @_;
+	
+	my @entries =
+		map   { values %{$self->{entries}{$package}} } 
+		grep  { $_ eq $package } 
+		keys %{ $self->{entries} };
+	}
+
+=item get_entries_by_distribution( DISTRIBUTION )
+
+Returns the entry objects for the named DISTRIBUTION.
+
+=cut
+
+sub get_entries_by_distribution
+	{
+	require CPAN::DistnameInfo;
+	my( $self, $distribution ) = @_;
+	croak "You must specify a distribution!" unless defined $distribution;
+	
+	my @entries =
+		grep  { # $_ is the entry hash
+			my $info = CPAN::DistnameInfo->new( $_->{'path'} );
+			defined $info->dist && $info->dist eq $distribution;
+			}
+		map { # $_ is the package name
+			values %{ $self->{entries}{$_} }
+			}
+		keys %{ $self->{entries} };
+	}
+
+=item get_entries_by_version( VERSION )
+
+Returns the entry objects for any entries with VERSION.
+
+=cut
+
+sub get_entries_by_version
+	{
+	my( $self, $version ) = @_;
+	
+	my @entries =
+		map   { $self->{entries}{$_}{$version} }
+		grep  { exists $self->{entries}{$_}{$version} } 
+		keys %{ $self->{entries} };
+	}
+
+=item get_entries_by_path( PATH )
+
+Returns the entry objects for any entries with PATH.
+
+=cut
+
+sub get_entries_by_path
+	{
+	my( $self, $path ) = @_;
+	
+	my @entries =
+		map   { $self->{entries}{$_}{$path} }
+		grep  { exists $self->{entries}{$_}{$path} } 
+		keys %{ $self->{entries} };
+	}
+
 =back
 
 =head1 TO DO
