@@ -6,10 +6,12 @@ use subs qw();
 use vars qw($VERSION);
 
 use Carp qw(carp croak cluck confess);
+use Cwd;
+use File::Basename;
 use File::Spec::Functions;
 
 BEGIN {
-	$VERSION = '0.21_03';
+	$VERSION = '0.25_03';
 	}
 
 =head1 NAME
@@ -108,17 +110,44 @@ and you try to add that package twice, the object will die. See C<add_entry>.
 
 =cut
 
+BEGIN {
+my $class_counter = 0;
 sub new
 	{
 	my( $class, %args ) = @_;
 
-	my $self = bless {}, $class;
+	my( $ref, $bless_class ) = do {
+		if( exists $args{dbmdeep} )
+			{
+			eval { require DBM::Deep };
+			if( $@ ) {
+				croak "You must have DBM::Deep installed and discoverable to use the dbmdeep feature";
+				}
+			my $ref = DBM::Deep->new( 
+				file => $args{dbmdeep}, 
+				autoflush => 1,
+				);
+			croak "Could not create DBM::Deep object" unless ref $ref;
+			my $single_class = sprintf "${class}::DBM%03d", $class_counter++;
+			
+			no strict 'refs';
+			@{"${single_class}::ISA"} = ( $class , 'DBM::Deep' );
+			( $ref, $single_class );
+			}
+		else
+			{
+			( {}, $class );
+			}
+		};
+		
+	my $self = bless $ref, $bless_class;
 	
 	$self->init( %args );
 	
 	$self;
 	}
-	
+}
+
 =item init
 
 Sets up the object. C<new> calls this automatically for you.
@@ -146,7 +175,7 @@ BEGIN {
 # them to the right delegate
 my %Dispatch = (
 		header  => { map { $_, 1 } qw(default_headers get_header set_header header_exists columns_as_list) },
-		entries => { map { $_, 1 } qw(add_entry count as_unique_sorted_list already_added allow_packages_only_once) },
+		entries => { map { $_, 1 } qw(add_entry count as_unique_sorted_list already_added allow_packages_only_once get_entries_by_package get_entries_by_version get_entries_by_path get_entries_by_distribution) },
 	#	entry   => { map { $_, 1 } qw() },
 		);
 		
@@ -232,16 +261,18 @@ sub init
 		{
 		eval "require $class";
 		}
-	
+
+	# don't initialize things if they are already there. For instance, 
+	# if we read an existing DBM::Deep file
 	$self->{entries} = $self->entries_class->new(
 		entry_class              => $self->entry_class,
 		columns                  => [ split /,\s+/, $config{columns} ],
 		allow_packages_only_once => $config{allow_packages_only_once},
-		);
+		) unless exists $self->{entries};
 	
 	$self->{header}  = $self->header_class->new(
 		_entries => $self->entries,
-		);
+		) unless exists $self->{header};
 	
 	
 	foreach my $key ( keys %config )
@@ -282,8 +313,9 @@ sub read
 		
 	require IO::Uncompress::Gunzip;
 
-	my $fh = IO::Uncompress::Gunzip->new( $file ) or do {	
-		carp "Could not open $file: $IO::Compress::Gunzip::GunzipError";
+	my $fh = IO::Uncompress::Gunzip->new( $file ) or do {
+		no warnings;
+		carp "Could not open $file: $IO::Compress::Gunzip::GunzipError\n";
 		return;
 		};
 	
@@ -389,126 +421,301 @@ sub write_fh
 	print $fh $self->header->as_string, $self->entries->as_string;
 	}
 	
-=item check_file
+=item check_file( FILE, CPAN_PATH )
 
+This method takes an existing F<02packages.details.txt.gz> named in FILE and
+the the CPAN root at CPAN_PATH (to append to the relative paths in the
+index), then checks the file for several things:
+
+	1. That there are entries in the file
+	2. The number of entries matches those declared in the Line-Count header
+	3. All paths listed in the file exist under CPAN_PATH
+	4. All distributions under CPAN_PATH have an entry (not counting older versions)
+
+If any of these checks fail, C<check_file> croaks with a hash reference
+with these keys:
+
+	# present in every error object
+	filename                the FILE you passed in
+	cpan_path               the CPAN_PATH you passed in
+	cwd                     the current working directory
+	error_count             
+
+	# if FILE is missing
+	missing_file          exists and true if FILE doesn't exist
+
+	# if the entry count in the file is wrong
+	# that is, the actual line count and header disagree
+	entry_count_mismatch    true 
+	line_count              the line count declared in the header   
+	entry_count             the actual count
+
+	# if some distros in CPAN_HOME are missing in FILE		
+	missing_in_file         anonymous array of missing paths
+	
+	# if some entries in FILE are missing the file in CPAN_HOME
+	missing_in_repo         anonymous array of missing paths
 
 =cut
 
+sub ENTRY_COUNT_MISMATCH () { 1 }
+sub MISSING_IN_REPO      () { 2 }
+sub MISSING_IN_FILE      () { 3 }
+
 sub check_file
 	{
-	my( $class, $file, $cpan_path ) = @_;
+	my( $either, $file, $cpan_path ) = @_;
 
+	# works with a class or an instance. We have to create a new
+	# instance, so we need the class. However, I'm concerned about
+	# subclasses, so if the higher level application just has the
+	# object, and maybe from a class I don't know about, they should
+	# be able to call this method and have it end up here if they
+	# didn't override it. That is, don't encourage them to hard code 
+	# a class name
+	my $class = ref $either || $either;
+	
 	# file exists
-	croak( "check_file is a class method, but you called it on an instance [$class]" ) if ref $class;
-	croak( "File [$file] does not exist!\n" ) unless -e $file;
-
+	my $error = { 
+		error_count => 0, 
+		cpan_path   => $cpan_path, 
+		filename    => $file,
+		cwd         => cwd(),
+		};
+	unless( -e $file )
+		{
+		$error->{missing_file}         = 1;
+		$error->{error_count}         +=  1;
+		}
+		
 	# file is gzipped
 
 	# check header # # # # # # # # # # # # # # # # # # #
 	my $packages = $class->read( $file );
 	
 	# count of entries in non-zero # # # # # # # # # # # # # # # # # # #
+
 	my $header_count = $packages->get_header( 'line_count' );
-	croak( "The header says there are no entries!\n" ) 
-		if $header_count == 0;
-		
-	# count of lines matches # # # # # # # # # # # # # # # # # # #
 	my $entries_count = $packages->count;
-
-	croak( "Entry count mismatch? " .
-		"The header says $header_count but there are only $entries_count records\n" )
-		unless $header_count == $entries_count;
-
-	# all listed distributions are in repo # # # # # # # # # # # # # # # # # # #
-	my @missing;
-	if( defined $cpan_path )
+	
+	unless( $header_count )
 		{
-		croak( "CPAN path [$cpan_path] does not exist!\n" ) unless -e $cpan_path;
-		
-		# this entries syntax really sucks
-		my( $entries ) = $packages->as_unique_sorted_list;
-		foreach my $entry ( @$entries )
-			{
-			my $path = $entry->path;
-			
-			my $native_path = catfile( $cpan_path, split m|/|, $path );
-			
-			push @missing, $path unless -e $native_path;
-			}
-			
-		croak( 
-			"Some paths in $file do not show up under $cpan_path\n" .
-			join( "\n\t", @missing ) . "\n" 
-			)
-			if @missing;
-			
+		$error->{entry_count_mismatch} = 1;
+		$error->{line_count}           = $header_count;
+		$error->{entry_count}          = $entries_count;
+		$error->{error_count}         +=  1;
 		}
 
-	# all repo distributions are listed # # # # # # # # # # # # # # # # # # #
-	# the trick here is to not care about older versions
-	if( defined $cpan_path )
+	unless( $header_count == $entries_count )
 		{
-		croak( "CPAN path [$cpan_path] does not exist!\n" ) unless -e $cpan_path;
-			
-		my $dists = $class->_get_repo_dists( $cpan_path );
-		
-		$class->_filter_older_dists( $dists );
-		
-		my %files = map { $_, 1 } @$dists;
-		
-		my( $entries ) = $packages->as_unique_sorted_list;
-
-		foreach my $entry ( @$entries )
-			{
-			my $path = $entry->path;
-			
-			my $native_path = catfile( $cpan_path, split m|/|, $path );
-			
-			delete $files{$native_path};
-			}
-
-		croak( 
-			"Some paths in $cpan_path do not show up in $file\n" .
-			join( "\n\t", keys %files ) . "\n" 
-			)
-			if keys %files;
-		
+		$error->{entry_count_mismatch} = 1;
+		$error->{line_count}           = $header_count;
+		$error->{entry_count}          = $entries_count;
+		$error->{error_count}         +=  1;
 		}
+
+	if( $cpan_path )
+		{
+		my $missing_in_file = $packages->check_for_missing_dists_in_file( $cpan_path );
+		my $missing_in_repo = $packages->check_for_missing_dists_in_repo( $cpan_path );
 		
+		$error->{missing_in_file}  =  $missing_in_file if @$missing_in_file;
+		$error->{missing_in_repo}  =  $missing_in_repo if @$missing_in_repo;
+		$error->{error_count}     += @$missing_in_file  + @$missing_in_repo;
+		}
+
+	croak $error if $error->{error_count};
+	
 	return 1;
 	}
 
+
+
+=item check_for_missing_dists_in_repo( CPAN_PATH )
+
+Given an object and a CPAN_PATH, return an anonymous array of the 
+distributions in the object that are not in CPAN_PATH. That is,
+complain when the object has extra distributions.
+
+C<check_file> calls this for you and adds the result to its
+error output.
+
+=cut
+
+sub check_for_missing_dists_in_repo
+	{
+	my( $packages, $cpan_path ) = @_;
+
+	my @missing;
+	my( $entries ) = $packages->as_unique_sorted_list;
+	foreach my $entry ( @$entries )
+		{
+		my $path = $entry->path;
+		
+		my $native_path = catfile( $cpan_path, split m|/|, $path );
+		
+		push @missing, $path unless -e $native_path;
+		}
+		
+	return \@missing;
+	}
+
+=item check_for_missing_dists_in_file( CPAN_PATH )
+
+Given an object and a CPAN_PATH, return an anonymous array of the 
+distributions in CPAN_PATH that do not show up in the object. That is,
+complain when the object doesn't have all the dists.
+
+C<check_file> calls this for you and adds the result to its
+error output.
+
+=cut
+
+sub check_for_missing_dists_in_file
+	{
+	my( $packages, $cpan_path ) = @_;
+	
+	my $dists = $packages->_get_repo_dists( $cpan_path );
+	
+	$packages->_filter_older_dists( $dists );
+	
+	my %files = map { $_, 1 } @$dists;
+	use Data::Dumper;
+	
+	my( $entries ) = $packages->as_unique_sorted_list;
+
+	foreach my $entry ( @$entries )
+		{
+		my $path = $entry->path;
+		my $native_path = catfile( $cpan_path, split m|/|, $path );			
+		delete $files{$native_path};
+		}
+	
+	[ keys %files ];
+	}
+
 sub _filter_older_dists
 	{
 	my( $self, $array ) = @_;
-
-	require CPAN::DistnameInfo;
 	
 	my %Seen;
-	
-	my @temp = map { # extract for the hash
-			$_->[0]
+	my @order;
+	require  CPAN::DistnameInfo;
+	foreach my $path ( @$array )
+		{
+		my( $basename, $directory, $suffix ) = fileparse( $path, qw(.tar.gz .tgz .zip .tar.bz2) );
+		my( $name, $version, $developer ) = CPAN::DistnameInfo::distname_info( $basename );
+		my $tuple = [ $path, $name, $version ];
+		push @order, $name;
+		
+		   # first branch, haven't seen the distro yet
+		   if( ! exists $Seen{ $name } )        { $Seen{ $name } = $tuple }
+		   # second branch, the version we see now is greater than before
+		elsif( $Seen{ $name }[2] lt $version )  { $Seen{ $name } = $tuple }
+		   # third branch, nothing. Really? Are you sure there's not another case?
+		else                                   { () }
+		}
+		
+	@$array = map { 
+		if( exists $Seen{$_} )
+			{
+			my $dist = $Seen{$_}[0];
+			delete $Seen{$_};
+			$dist;
 			}
-		map { 
-			   # first branch, we've never seen this distro name
-			   if( ! exists $Seen{ $_->[1] } )     { $Seen{$_->[1]} = $_ }
-			   # second branch, the version we see now is greater than before
-			elsif( $Seen{ $_->[1] }[2] < $_->[2] ) { $Seen{$_->[1]} = $_ }
-			   # third branch, nothing. Really? Are you sure there's not another case?
-			else                                   { () }
+		else
+			{
+			()
 			}
-		map { # create the tuples
-			my $path = $_;
-			my $distname = CPAN::DistnameInfo->new($path);
-			my( $name, $version ) = map { $distname->$_ } qw(dist version);
-			[ $path, $name, $version ];
-			} @$array;
-	
-	@$array = @temp;
+		} @order;
 	
 	return 1;
 	}
+
+
+sub _distname_info 
+	{
+	my $file = shift or return;
 	
+	my ($dist, $version) = $file =~ /^
+		(                          # start of dist name
+			(?:
+				[-+.]*
+
+				(?:
+					[A-Za-z0-9]+
+						|
+					(?<=\D)_
+						|
+					_(?=\D)
+				)*
+	 			
+	 			(?:
+					[A-Za-z]
+					(?=
+						[^A-Za-z]
+						|
+						$
+					)
+						|
+					\d
+					(?=-)
+	 			)
+	 			
+	 			(?<!
+	 				[._-][vV]
+	 			)
+			)+
+		)                          # end of dist name
+
+		(                          # start of version
+		.*
+		)                          # end of version
+	$/xs or return ($file, undef, undef );
+
+	$dist =~ s/-undef\z// if ($dist =~ /-undef\z/ and ! length $version);
+
+	# Catch names like Unicode-Collate-Standard-V3_1_1-0.1
+	# where the V3_1_1 is part of the distname
+	if ($version =~ /^(-[Vv].*)-(\d.*)/) {
+		$dist    .= $1;
+		$version  = $2;
+		}
+
+	$version = $1            if !length $version and $dist =~ s/-(\d+\w)$//;
+
+	$version = $1 . $version if $version =~ /^\d+$/ and $dist =~ s/-(\w+)$//;
+
+	if( $version =~ /\d\.\d/ ) { $version =~ s/^[-_.]+// }
+	else                       { $version =~ s/^[-_]+//  }
+
+	# deal with versions with extra information
+	$version =~ s/-build\d+.*//;
+	$version =~ s/-DRW.*//;
+	
+	# deal with perl versions, merely to see if it is a dev version
+	my $dev;
+	if( length $version ) 
+		{
+		$dev = do {
+			if ($file =~ /^perl-?\d+\.(\d+)(?:\D(\d+))?(-(?:TRIAL|RC)\d+)?$/) 
+				{
+				 1 if (($1 > 6 and $1 & 1) or ($2 and $2 >= 50)) or $3;
+				}
+			elsif ($version =~ /\d\D\d+_\d/) 
+				{
+				1;
+				}
+			};
+		}
+	else 
+		{
+		$version = undef;
+		}
+
+	($dist, $version, $dev);
+	}
+
 sub _get_repo_dists
 	{	
 	my( $self, $cpan_home ) = @_;
@@ -562,7 +769,6 @@ sub header { $_[0]->{header} }
 
 =cut
 
-
 =back
 	
 =head2 Entries
